*******************************************************************************
*filename:  MC11MDSM.ASM
*REVISION HISTORY:
*
*DATE		REV. NO.	DESCRIPTION
*
* 8/14/99	1.0		Initial release
*
*
*******************************************************************************
*TECHNOLOGICAL ARTS
*MicroCore-11 Motor Driver Stepper Motor Demo
*14 AUGUST 1999
*by Exequiel Rarama

* System Addresses

RAM		equ	$0000
rbase		equ	$1000
STACK		equ	$01FF
EEPROM		equ	$e000			;assume an 8K code space


	org	RAM


state_timer             rmb     2               ;general purpose state timer
state                   rmb     2

;Stepper variables

stepper_timer           rmb     2
stepper_state           rmb     2


* registers

**********************************************************************
*	PORT ASSIGNMENTS for this program

porta   equ     0               ;input/output port

*	PA0	spare		;input only
*	PA1	spare		;input only
*	PA2	spare		;input only
*	PA3	spare		;MOTOR A ENABLE (define as output)
*	PA4	spare		;MOTOR B ENABLE (output only)
*	PA5	spare		;MOTOR A DIR (output only)
*	PA6	SPEAKER		;MOTOR B DIR (output only)
*	PA7	spare		;input or output
**********************************************************************

pioc    equ     $02             ;parallel i/o control register
portd   equ     $08             ;portd, serial communications
ddrd    equ     $09             ;data direction register, portd

oc1m    equ     $0c             ;output compare 1 mask register, porta
tcnt    equ     $0e             ;timer count
toc1    equ     $16             ;output compare register 1 
toc2    equ     $18             ;output compare register 2
toc3    equ     $1A             ;output compare register 3
toc4    equ     $1C             ;output compare register 4
tctl1   equ     $20             ;timer control register
tmsk1   equ     $22             ;OC/IC interrupt mask register
tflg1   equ     $23             ;main timer interrupt flags
tmsk2   equ     $24             ;timer interrupt mask register, prescaler
tflg2   equ     $25             ;timer interrupt flag register 2
pactl   equ     $26             ;real time interrupt prescaler, pulse accum.
baud    equ     $2b             ;sci baud rate control
sccr1   equ     $2c             ;sci control register 1
sccr2   equ     $2d             ;sci control register 2
scsr    equ     $2e             ;sci status register
scdr    equ     $2f             ;sci data, read (rdr) and write (tdr)

*Analog-to-Digital Converter

adctl   equ     $30             ;a/d control register
adr1    equ     $31             ;a/d channel 1 register
adr2    equ     $32             ;a/d channel 2 register
adr3    equ     $33             ;a/d channel 3 register
adr4    equ     $34             ;a/d channel 4 register

option  equ     $39             ;system options   

hprio   equ     $3c             ;highest priority interrupt, mode control

* Masks

opts  	equ	%00010000	;default (a/d off)
tmask	equ	%01000000	;(tmsk2)no timer prescale, RTI enabled
rtimsk	equ	%00001000	;(pactl) 4.1 msec interrupt with 8 MHz xtal
tc1msk	equ	%01000000	;OC2 in toggle mode

clrmsk	equ	%11000000	;mask for clearing timer flags
baud96 equ      %00110000       ;(baud) 9600 baud with 8 MHz crystal
baud24 equ      %00110010       ;(baud) 2400 baud with 8 MHz crystal
baud12 equ      %00110011       ;(baud) 1200 baud with 8 MHz crystal
ptdmsk	equ 	%00000010	;(ddrd)portd data direction: input, except txd
sc2msk2	equ	%00101100	;(sccr2)enable tx & rx and Rx Interrupt

;-----------------------------------------------------------------------------
;Port A pin assignment
;PA3 - Enable signal for EN1 and EN2
;PA4 - Enable signal for EN3 and EN4
;PA5 - Motor direction for IN1 and IN2
;PA6 - Motor direction for IN3 and IN4

enable1_2       equ     %0001000        ;0 - Disable, 1 - Enable
enable3_4       equ     %0010000        ;0 - Disable, 1 - Enable
d_init          equ     %1100000
kill_stepper    equ     %0011000
DDRA3           equ     %1000

********************************** Program *******************************

	org	EEPROM

init:
	sei			;this is where the RESET vector points
        ldx     #rbase

*NOTE-  These instructions must be executed within 64 E-cycles of reset.

	ldaa	#opts		;define system options
        staa    option,x

	lds	#STACK	 	;initialize stack pointer

	ldaa	#tmask		;enable real-time interrupt (RTI)
        staa    tmsk2,x

	ldaa	#rtimsk		;initialize RTI rate
        staa    pactl,x

	jsr	stepper_init	;initialize the port lines and states for the stepper

	cli			;enable interrupts

;Main Loop

begin:
        ldx     stepper_state
        jsr     0,x
        bra     begin

;Stepping the stepper motor
stepper_init
        ldx     #rbase
        bclr    porta,x #kill_stepper
        bclr    porta,x #d_init
        bset    pactl,x #DDRA3                  ;Make bit 3 of Port A output

        ldx     #step_forward2
        stx     stepper_state

        ldy     #1				;stepper_timer is decremented by RTI
        sty     stepper_timer

        ldx     #rbase
        bset    porta,x #%01100000

        nop
        nop

        bset    porta,x enable1_2
        bset    porta,x enable3_4

        rts

step_forward1
        ldx     stepper_timer
        bne     step_ex

        ldx     #rbase
        bset    porta,x #%01100000

        ldx     #step_forward2
        stx     stepper_state

        ldx     #1
        stx     stepper_timer

        bra     step_ex

step_forward2
        ldx     stepper_timer
        bne     step_ex

        ldx     #rbase
        bclr    porta,x #%01000000

        ldx     #step_backward1
        stx     stepper_state

        ldx     #1
        stx     stepper_timer

        bra     step_ex

step_backward1
        ldx     stepper_timer
        bne     step_ex

        ldx     #rbase
        bclr    porta,x #%01100000

        ldx     #step_backward2
        stx     stepper_state

        ldx     #1
        stx     stepper_timer

        bra     step_ex

step_backward2
        ldx     stepper_timer
        bne     step_ex

        ldx     #rbase
        bset    porta,x #%01000000

        ldx     #step_forward1
        stx     stepper_state

        ldx     #1
        stx     stepper_timer

        bra     step_ex

step_ex
        rts


*******************************************************************************
*  Interrupt Routines
*******************************************************************************

* Real-time Interrupt Routine

RealTimeInt:

RTI_3
        ldx     state_timer
        beq     RTI_4
        dex     state_timer
        stx     state_timer
RTI_4
        ldx     stepper_timer
        beq     RTI_5
        dex     stepper_timer
        stx     stepper_timer
RTI_5
        ldaa    #%01000000      ;clear real-time interrupt flag
        ldy     #rbase
        staa    tflg2,y
	rti

*******************************************************************************

	org	$FFD6

*  Interrupt and reset vectors.

SCI_VECT	FDB	init
SPI_VECT	FDB	init
PAI_VECT	FDB	init
PAO_VECT	FDB	init
TOF_VECT	FDB	init

TOC5_VECT	FDB	init
TOC4_VECT	FDB	init
TOC3_VECT	FDB	init
TOC2_VECT	FDB	init
TOC1_VECT	FDB	init
TIC3_VECT	FDB	init
TIC2_VECT	FDB	init
TIC1_VECT	FDB	init

RTI_VECT	FDB	RealTimeInt
IRQ_VECT	FDB	init
XIRQ_VECT	FDB	init
SWI_VECT	FDB	init
TRAP_VECT	FDB	init
COP_FAIL_VECT	FDB	init
COP_CMF_VECT	FDB	init
RESET_VECT	FDB	init
